#include <iostream>
#include <fstream>
#include <vector>

using namespace std;

const int inf = 0x3f3f3f3f;

class Graph {
  public:
    Graph(const int &size = 1):
        edges_(size) {
    }

    void addEdge(const int &firstNode, const int &secondNode) {
        edges_[firstNode].push_back(secondNode);
        edges_[secondNode].push_back(firstNode);
    }

    void liniarize() {
        begin_ = end_ = vector<int>(edges_.size());
        liniarize_ = vector<int>(edges_.size() * 2);
        time_ = 0;

        dfs(0);
    }

    const int & start(const int &node) const {
        return begin_[node];
    }

    const int & end(const int &node) const {
        return end_[node];
    }

    int get(const int &position) const {
        if (position == -inf)
            return -2;
        return liniarize_[position];
    }

  private:
    void dfs(const int &node, const int &father = -1) {
        begin_[node] = time_;
        liniarize_[time_++] = node;

        for (vector<int>::iterator it = edges_[node].begin(); it != edges_[node].end(); ++it)
            if (*it != father)
                dfs(*it, node);

        end_[node] = time_;
        liniarize_[time_++] = node;
    }

    vector< vector<int> > edges_;

    int time_;
    vector<int> liniarize_, begin_, end_;
};

class SegmentTree {
  public:
    SegmentTree(const int &size_) {
        for (size = 1; size < size_; size *= 2);

        sum_ = best_ = vector<int>(size * 2, 0);
    }

    void add(int position, const int &value) {
        position += size;
        best_[position] = (sum_[position] += value);
        
        for (position /= 2; position; position /= 2) {
            sum_[position] += value;
            best_[position] = max(best_[position * 2], sum_[position * 2] + best_[position * 2 + 1]);
        }

        //add(1, 0, size, position, value);
    }

    int query(const int &start, const int &finish) {
        int value = sum(start, finish);
        if (value == 0)
            return -inf;
        tempsum = 0;
        return query(start, finish, value);
        return query(1, 0, size, start, finish, value);
    }

  private:
    void add(const int &node, const int &begin, const int &end, const int &position, const int &value) {
        if (end - begin == 1) {
            sum_[node] += value;
            best_[node] = sum_[node];
            return;
        }

        int mid = (begin + end) / 2;
        if (position < mid)
            add(node * 2, begin, mid, position, value);
        else
            add(node * 2 + 1, mid, end, position, value);

        sum_[node] += value;
        best_[node] = max(best_[node * 2], sum_[node * 2] + best_[node * 2 + 1]);
    }

    int sum(int from, int to) {
        int answer = 0;
        from += size;
        to += size;
        while (from <= to) {
            if (from % 2)
                answer += sum_[from];
            if (to % 2 == 0)
                answer += sum_[to];

            from = (from + 1) / 2;
            to = (to - 1) / 2;
        }

        return answer;
    }

    int find(int node, int value) {
        for (; node < size;) {
            if (best_[node * 2] >= value)
                node = node * 2;
            else {
                value -= sum_[node * 2];
                node = node * 2 + 1;
            }
        }

        return node - size;
    }

    int query(int from, int to, int value) {
        int newFrom = from + size;
        int newTo = to + size;

        int dir = 0;
        while (newFrom <= newTo) {
            if (newFrom % 2) {
                if (best_[newFrom] >= value)
                    return find(newFrom, value);
                value -= sum_[newFrom];
            }

            dir = dir * 2 + newTo % 2;
            newFrom = (newFrom + 1) / 2;
            newTo = (newTo - 1) / 2;
        }

        do {
            newTo = newTo * 2  + 2 - dir % 2;

            if (dir % 2 == 0) {
                if (best_[newTo] >= value)
                    return find(newTo, value);
                value -= sum_[newTo];
            }

            dir /= 2;
        } while (newTo != to);

        return -inf;
    }

    int query(const int &node, const int &begin, const int &end, const int &start, const int &finish, const int &value) {
        if (start <= begin && end - 1 <= finish) {
            if (best_[node] + tempsum < value) {
                tempsum += sum_[node];
                return -inf;
            }

            if (end - begin == 1) {
                tempsum += sum_[node];
                return begin;
            }
        }

        int mid = (begin + end) / 2;

        if (start >= mid)
            return query(node * 2 + 1, mid, end, start, finish, value);
        if (finish < mid)
            return query(node * 2, begin, mid, start, finish, value);

        // if it's on the left size
        int answer = query(node * 2, begin, mid, start, finish, value);
        if (answer != -inf)
            return answer;
        // else
        answer = query(node * 2 + 1, mid, end, start, finish, value);

        if (start <= begin && end - 1 <= finish)
            tempsum += sum_[node];
        return answer;
    }

    int size;
    int tempsum;
    vector<int> sum_, best_;
};

int main() {
    ifstream cin("confuzie.in");
    ofstream cout("confuzie.out");

    int N, M; cin >> N >> M;
    
    Graph G(N);
    for (int i = 1; i < N; ++i) {
        int x, y; cin >> x >> y;
        G.addEdge(x - 1, y - 1);
    }
    
    G.liniarize();

    SegmentTree S(2 * N);
    vector<bool> value(N, false);
    for (int i = 0; i < M; ++i) {
        int type, node;
        cin >> type >> node;

        if (type == 0) {
            --node;
            value[node] = !value[node];
            if (value[node]) {
                S.add(G.start(node), -1);
                S.add(G.end(node), 1);
            } else {
                S.add(G.start(node), 1);
                S.add(G.end(node), -1);
            }
        } else {
            int second; cin >> second;
            cout << G.get(S.query(G.end(second - 1), G.end(node - 1))) + 1 << endl;
        }
    }
}

